import 'dart:async';

import 'package:flutterx_http/flutterx_http.dart';
import 'package:http/http.dart';

class TransformRequest extends BaseRequest implements CloneableRequest {
  final BaseRequest source;
  final StreamTransformer<List<int>, List<int>> transform;

  TransformRequest({String? method, Uri? url, required this.source, required this.transform})
      : super(method ?? source.method, url ?? source.url) {
    contentLength = source.contentLength;
    persistentConnection = source.persistentConnection;
    followRedirects = source.followRedirects;
    maxRedirects = source.maxRedirects;
    headers.addAll(source.headers);
  }

  @override
  ByteStream finalize() {
    super.finalize();
    return ByteStream(source.finalize().transform(transform));
  }

  @override
  TransformRequest copy({
    String? method,
    Uri? url,
    BaseRequest? source,
    StreamTransformer<List<int>, List<int>>? transform,
    bool? persistentConnection,
    bool? followRedirects,
    int? maxRedirects,
    Map<String, String>? headers,
  }) =>
      TransformRequest(
          method: method ?? this.method,
          url: url ?? this.url,
          source: source ?? this.source,
          transform: transform ?? this.transform)
        ..persistentConnection = persistentConnection ?? this.persistentConnection
        ..followRedirects = followRedirects ?? this.followRedirects
        ..maxRedirects = maxRedirects ?? this.maxRedirects
        ..headers.addAll(headers ?? this.headers);
}
